@@ -79,16 +79,6 @@ test_that("convert to vector works for data.frame", {
   )
 })
 
-test_that("convert to vector works for partial_frame", {
-  array <- as_nanoarrow_array(
-    data.frame(a = 1L, b = "two", stringsAsFactors = FALSE)
-  )
-  expect_identical(
-    convert_array(array, vctrs::partial_frame()),
-    data.frame(a = 1L, b = "two", stringsAsFactors = FALSE)
-  )
-})
-
 test_that("convert to vector works for extension<struct> -> data.frame()", {
   array <- nanoarrow_extension_array(
     data.frame(x = c(TRUE, FALSE, NA, FALSE, TRUE)),
@@ -115,6 +105,8 @@ test_that("convert to vector works for dictionary<struct> -> data.frame()", {
 })
 
 test_that("convert to vector works for function()", {
+  skip_if_not_installed("tibble")
+
   tibble_or_bust <- function(array, ptype) {
     if (is.data.frame(ptype)) {
       ptype <- tibble::as_tibble(ptype)
@@ -135,6 +127,8 @@ test_that("convert to vector works for function()", {
 })
 
 test_that("convert to vector works for tibble", {
+  skip_if_not_installed("tibble")
+
   array <- as_nanoarrow_array(
     data.frame(a = 1L, b = "two", stringsAsFactors = FALSE)
   )
@@ -160,6 +154,17 @@ test_that("convert to vector works for tibble", {
   )
 })
 
+test_that("convert to vector works for a rcrd-sytle vctr with complex columns", {
+  skip_if_not_installed("vctrs")
+
+  rcrd <- vctrs::new_rcrd(list(x = data.frame(y = 1:10)))
+  rcrd_array <- as_nanoarrow_array(vctrs::vec_data(rcrd))
+  expect_identical(
+    convert_array(rcrd_array, rcrd),
+    rcrd
+  )
+})
+
 test_that("convert to vector works for nanoarrow_vctr()", {
   array <- as_nanoarrow_array(c("one", "two", "three"))
 
@@ -293,6 +298,8 @@ test_that("convert to vector works for struct-style vectors", {
 })
 
 test_that("convert to vector works for unspecified()", {
+  skip_if_not_installed("vctrs")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -605,22 +612,31 @@ test_that("convert to vector works for valid double()", {
   )
 })
 
-test_that("convert to vector works for decimal128 -> double()", {
+test_that("convert to vector works for decimal -> double()", {
+  constructors <- list(na_decimal32, na_decimal64, na_decimal256, na_decimal128)
+  for (constructor in constructors) {
+    numbers <- round(c(pi, -pi, 123.4567, NA, -123.4567, 0, 123), 4)
+
+    # Check scale of 4 (min required for lossless roundtrip)
+    array4 <- as_nanoarrow_array(numbers, schema = constructor(9, 4))
+    expect_identical(convert_array(array4), numbers)
+
+    # Check scale of 5 (requires adding some zeroes in C)
+    array5 <- as_nanoarrow_array(numbers, schema = constructor(9, 5))
+    expect_identical(convert_array(array5), numbers)
+
+    # Check negative scale (also requires adding some zeroes in C)
+    numbers_neg_scale <- c(12300, -12300, 0, NA, 100)
+    array_neg2 <- as_nanoarrow_array(numbers_neg_scale, schema = constructor(9, -2))
+    expect_identical(convert_array(array_neg2), numbers_neg_scale)
+  }
+
+  # Make sure we agree with arrow
   skip_if_not_installed("arrow")
 
-  array <- as_nanoarrow_array(arrow::Array$create(1:10)$cast(arrow::decimal128(20, 10)))
-
-  # Check via S3 dispatch
-  expect_equal(
-    convert_array(array, double()),
-    as.double(1:10)
-  )
-
-  # ...and via C -> S3 dispatch
-  expect_equal(
-    convert_array.default(array, double()),
-    as.double(1:10)
-  )
+  expect_identical(as.vector(arrow::as_arrow_array(array4)), numbers)
+  expect_identical(as.vector(arrow::as_arrow_array(array5)), numbers)
+  expect_identical(as.vector(arrow::as_arrow_array(array_neg2)), numbers_neg_scale)
 })
 
 test_that("convert to vector works for null -> double()", {
@@ -864,6 +880,37 @@ test_that("convert to vector works for dictionary<string> -> factor()", {
   )
 })
 
+test_that("convert to vector works for decimal -> character()", {
+  constructors <- list(na_decimal32, na_decimal64, na_decimal256, na_decimal128)
+  for (constructor in constructors) {
+    numbers <- round(c(pi, -pi, 123.4567, NA, -123.4567, 0, 123), 4)
+
+    # Check scale of 4 (min required for lossless roundtrip)
+    array4 <- as_nanoarrow_array(numbers, schema = constructor(9, 4))
+    expect_identical(
+      convert_array(array4, character()),
+      c("3.1416", "-3.1416", "123.4567", NA_character_,
+        "-123.4567", "0.0000", "123.0000")
+    )
+
+    # Check scale of 5 (requires adding some zeroes in C)
+    array5 <- as_nanoarrow_array(numbers, schema = constructor(9, 5))
+    expect_identical(
+      convert_array(array5, character()),
+      c("3.14160", "-3.14160", "123.45670", NA_character_,
+        "-123.45670", "0.00000", "123.00000")
+    )
+
+    # Check negative scale (also requires adding some zeroes in C)
+    numbers_neg_scale <- c(12300, -12300, 0, NA, 100)
+    array_neg2 <- as_nanoarrow_array(numbers_neg_scale, schema = constructor(9, -2))
+    expect_identical(
+      convert_array(array_neg2, character()),
+      c("12300", "-12300", "000", NA_character_, "100")
+    )
+  }
+})
+
 test_that("batched convert to vector works for dictionary<string> -> factor()", {
   # A slightly different path: convert_array.factor() called from C multiple
   # times with different dictionaries each time.
@@ -925,6 +972,8 @@ test_that("convert to vector works for binary_view -> blob::blob()", {
 })
 
 test_that("convert to vector works for null -> blob::blob()", {
+  skip_if_not_installed("blob")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -937,6 +986,7 @@ test_that("convert to vector works for null -> blob::blob()", {
 
 test_that("convert to vector works for list -> vctrs::list_of", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   array_list <- as_nanoarrow_array(
     arrow::Array$create(
@@ -974,6 +1024,7 @@ test_that("convert to vector works for list -> vctrs::list_of", {
 
 test_that("convert to vector works for large_list -> vctrs::list_of", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   array_list <- as_nanoarrow_array(
     arrow::Array$create(
@@ -1003,6 +1054,7 @@ test_that("convert to vector works for large_list -> vctrs::list_of", {
 
 test_that("convert to vector works for fixed_size_list -> vctrs::list_of", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   array_list <- as_nanoarrow_array(
     arrow::Array$create(
@@ -1031,6 +1083,8 @@ test_that("convert to vector works for fixed_size_list -> vctrs::list_of", {
 })
 
 test_that("convert to vector works for null -> vctrs::list_of()", {
+  skip_if_not_installed("vctrs")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -1041,6 +1095,55 @@ test_that("convert to vector works for null -> vctrs::list_of()", {
   )
 })
 
+test_that("convert to vector works for map -> vctrs::list_of", {
+  skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
+
+  values <- vctrs::list_of(
+    data.frame(key = "key1", value = 1L),
+    data.frame(key = c("key2", "key3"), value = c(2L, 3L)),
+    NULL
+  )
+
+  array_list <- as_nanoarrow_array(
+    arrow::Array$create(
+      values,
+      type = arrow::map_of(arrow::string(), arrow::int32())
+    )
+  )
+
+  # Default conversion
+  expect_identical(
+    convert_array(array_list),
+    values
+  )
+})
+
+test_that("convert to vector works for fixed_size_list_of() -> matrix()", {
+  mat <- matrix(1:6, ncol = 2, byrow = TRUE)
+  array <- as_nanoarrow_array(mat)
+
+  expect_identical(
+    convert_array(array, matrix(double(), ncol = 2)),
+    matrix(as.double(1:6), ncol = 2, byrow = TRUE)
+  )
+})
+
+test_that("convert to vector errors for invalid matrix()", {
+  expect_error(
+    convert_array(as_nanoarrow_array(1:6), matrix()),
+    "Can't convert array <int32> to R vector of type matrix"
+  )
+
+  mat <- matrix(1:6, ncol = 2, byrow = TRUE)
+  array <- as_nanoarrow_array(mat)
+  expect_error(
+    convert_array(array, matrix(integer(), ncol = 3)),
+    "Can't convert fixed_size_list(list_size=2) to matrix with 3 cols",
+    fixed = TRUE
+  )
+})
+
 test_that("convert to vector works for Date", {
   array_date <- as_nanoarrow_array(as.Date(c(NA, "2000-01-01")))
   expect_identical(
@@ -1070,6 +1173,8 @@ test_that("convert to vector works for null -> Date", {
 })
 
 test_that("convert to vector works for hms", {
+  skip_if_not_installed("hms")
+
   array_time <- as_nanoarrow_array(hms::parse_hm("12:34"))
   expect_identical(
     convert_array(array_time),
@@ -1078,6 +1183,8 @@ test_that("convert to vector works for hms", {
 })
 
 test_that("convert to vector works for null -> hms", {
+  skip_if_not_installed("hms")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -1192,6 +1299,7 @@ test_that("convert to vector works for null -> difftime", {
 
 test_that("convert to vector works for data frames nested inside lists", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   df_in_list <- vctrs::list_of(
     data.frame(x = 1:5),
@@ -1208,6 +1316,7 @@ test_that("convert to vector works for data frames nested inside lists", {
 
 test_that("convert to vector works for lists nested in data frames", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   df_in_list_in_df <- data.frame(
     x = vctrs::list_of(
